#!/bin/bash

confDir=/etc/mistake-proof.d
mistakeProofConf=/etc/mistake-proof.conf
mpConfTmp=$(mktemp /tmp/XXXXXXXXXXXXXXXX)
if [[ $# -ge 1 ]]; then
    if [[ ! -d $1 ]]; then
        echo "Usage:"
        echo " sh 01-mistake-proof-conf.sh [confDir]"
        exit 1
    fi

    confDir=$1
    mistakeProofConf=$confDir/mistake-proof.conf
fi

export MISTAKE_PROOF_CONF=$confDir

cmdOptions=$confDir/mistake_proof_options
blacklist=$confDir/blacklist
mpBlackTmp=$(mktemp /tmp/XXXXXXXXXXXXXXXX)

mistakeProofAlias=$confDir/zzz_mistake_prool_02_alias.sh
mistakeProofUnalias=$confDir/zzz_mistake_prool_01_unalias.sh

function skipLine()
{

    if [[ x"$1" == x"" ]]; then
        return 0
    fi

    if [[ $1 =~ ^#.* ]]; then
        return 0
    fi

    return 1
}

echo "# Do not modify this file manually, it generates by 01-mistake-proof-conf.sh" > $mistakeProofUnalias
while read line
do
    if [[ x"$line" == x"" ]]; then
        continue
    fi

    if [[ $line =~ ^#.* ]]; then
        continue
    fi

    cmd=$(echo $line | awk -F "[ =]" '{print $2}')
    echo "unalias $cmd 1>/dev/null 2>&1" >> $mistakeProofUnalias
done < $mistakeProofAlias

declare -A cmdDic
cmdDic=()
cat $mistakeProofConf > $mpConfTmp
echo "" >> $mpConfTmp
cat $blacklist > $mpBlackTmp
echo "" >> $mpBlackTmp
while read line
do
    skipLine $line
    if [[ $? -eq 0 ]]; then
        continue
    fi

    line=$(echo "$line" | sed 's/ /#/g')
    cmd=${line%%#*}
    options=${line#*#}

    which $cmd 1>/dev/null 2>&1
    if [[ $? -ne 0 ]]; then
        echo "$cmd is not supported, ignored."
        continue
    fi

    isInBlacklist=0
    while read cmdBlacklist
    do
        skipLine $cmdBlacklist
        if [[ $? -eq 0 ]]; then
            continue
        fi

	if [[ $cmd == $cmdBlacklist ]]; then
            isInBlacklist=1
            break
        fi
    done < $mpBlackTmp

    if [[ $isInBlacklist -eq 1 ]]; then
        echo "$cmd is not supported, ignored."
        continue
    fi

    if [[ x"${cmdDic[${cmd}]}" == x"" ]]; then
        cmdDic[${cmd}]="(\"$options\")"
    else
        newValue="${cmdDic[${cmd}]%?} \"$options\")"
        cmdDic[${cmd}]=$newValue
    fi
done < $mpConfTmp

rm -rf $mpConfTmp $mpBlackTmp

echo "# Do not modify this file manually, it generates by 01-mistake-proof-conf.sh" > $mistakeProofAlias
echo "# Do not modify this file manually, it generates by 01-mistake-proof-conf.sh" > $cmdOptions
for key in $(echo ${!cmdDic[*]})
do
    realkey=$(alias $key 2>/dev/null | awk -F "'" '{print $2}')
    if [[ -z $realkey ]]; then
        realkey=$key
    else
        if [[ $realkey =~ "mistake_proof" ]]; then
            realkey=$(echo $realkey | awk -F "mistake_proof" '{print $3}' | awk -F "\"" '{print $1}' | awk -F "\"" '{print $1}' | sed -e 's/^[ ]*//g' | sed -e 's/[ ]*$//g')
	fi
    fi
    echo "alias $key='__mistakeProof(){ which mistake_proof 1>/dev/null 2>&1; if [[ \$? -eq 0 ]]; then command mistake_proof $realkey \"\$@\"; ret=\$?; else command $realkey \"\$@\"; ret=\$?; fi; return \$ret; }; __mistakeProof'" >> $mistakeProofAlias
    key_op=$(echo $key | sed s/-/_/g)
    realkey=$(echo "$realkey" | sed "s/${key}//g" | sed 's/ /#/g')
    if [[ -n "$realkey" ]]; then
        newValue=$(echo ${cmdDic[$key]} | sed "s/[( ]\"/&${realkey}#/g")
        cmdDic[${key}]=$newValue
    fi
    echo "__mistakeProof_${key_op}=${cmdDic[$key]}" >> $cmdOptions
done
